# Find duplicate files. (c) Peter Kankowski, 2009. http://www.strchr.com

import os, hashlib, sys, codecs, time
from os.path import join, getsize
DIRECTORY = 'c:\\Docum'
BUFFERSIZE = 64*1024

def raise_err(err):
    if type(err) == WindowsError and err.args[0] == 5: # ERROR_ACCESS_DENIED
        return
    raise err

# Return SHA-256 hash of a file
def hash_file(filename):
    try:
        file = open(filename, 'rb')
        hash = hashlib.sha256()
        while True:
            buff = file.read(BUFFERSIZE)
            if not buff:
                break
            hash.update(buff)
        return hash.digest()
    except IOError as err:
        if err.args[0] != 13: # Permission denied
            raise

# Print on Windows console (replacing non-encodable characters).
# See http://bugs.python.org/issue5110
def print_win(s):
    r = s.encode(sys.stdout.encoding, 'backslashreplace')
    print(codecs.getdecoder(sys.stdout.encoding)(r)[0])





starttime = time.clock()

# Find files with the same size
samesize = {}
for path, _, filenames in os.walk(DIRECTORY, onerror=raise_err):
    for filename in filenames:
        fullname = join(path, filename)
        samesize.setdefault(getsize(fullname), []).append(fullname)

# Calculate SHA-256 hashes for these files
for filenames in samesize.values():
    if len(filenames) > 1:
        hashes = {}
        for filename in filenames:
            hashes.setdefault(hash_file(filename), []).append(filename)
        
        # Print files with the same hash value
        for identicalfiles in hashes.values():
            if len(identicalfiles) > 1:
                print_win('\n'.join(identicalfiles) + '\n')

print( "{0:.2f} seconds".format(time.clock() - starttime) )
